{$WARN SYMBOL_PLATFORM OFF}
unit Unit2;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ComCtrls;

const
  BASS_CONFIG_V2_VIDEOWINDOW = $90001; //set video window
  BASS_CTYPE_V2_VIDEO = $90001; //channel type

  BASS_V2_ATTRIB_PLAY = $90001; //start/stop playback
  BASS_V2_ATTRIB_VIDEOSZ = $90003; //update window pos

type
  TForm2 = class(TForm)
    Button1: TButton;
    Panel1: TPanel;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    Timer1: TTimer;
    TrackBar1: TTrackBar;
    OpenDialog1: TOpenDialog;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
  private
  public
  end;

var
  Form2: TForm2;

implementation

uses
  bass, bassmix, basswasapi;

var
  MyDir: string;
  C: HCHANNEL;
  M: HCHANNEL; //for resampling

{$R *.dfm}

function Sec(B: Int64): Double;
begin
  Result := BASS_ChannelBytes2Seconds(C, B);
end;

procedure CheckBassError;
begin
  if BASS_ErrorGetCode <> 0 then
    raise Exception.CreateFmt('BASS: %d', [BASS_ErrorGetCode]);
end;

function WasapiProc(buffer:Pointer; length:DWORD; user:Pointer): DWORD; stdcall;
begin
  Result := BASS_ChannelGetData(M, buffer, length);
end;

procedure TForm2.Button1Click(Sender: TObject);
begin
  //open
  if OpenDialog1.Execute then
  begin
    BASS_StreamFree(C);
    C := BASS_StreamCreateFile(False, PChar(OpenDialog1.FileName), 0, 0, BASS_UNICODE or BASS_STREAM_DECODE);
    CheckBassError;
    BASS_Mixer_StreamAddChannel(M, C, 0);
    CheckBassError;
    TrackBar1.Max := BASS_ChannelGetLength(C, 0);
    TrackBar1.Position := 0;
  end;
end;

procedure TForm2.Button2Click(Sender: TObject);
begin
  //play
  BASS_ChannelSetAttribute(C, BASS_V2_ATTRIB_PLAY, 1); //start video
end;

procedure TForm2.Button3Click(Sender: TObject);
begin
  //stop
  BASS_ChannelSetAttribute(C, BASS_V2_ATTRIB_PLAY, 0); //stop video
end;

procedure TForm2.FormShow(Sender: TObject);
var
  WI: BASS_WASAPI_INFO;
begin
  //init
  BASS_Init(0, 44100, 0, Handle, nil);
  CheckBassError;
  BASS_PluginLoad(PChar(MyDir + 'bass_v2.dll'), BASS_UNICODE);
  CheckBassError;
  BASS_SetConfig(BASS_CONFIG_V2_VIDEOWINDOW, Panel1.Handle);
  CheckBassError;
  //wasapi playback
  BASS_WASAPI_Init(-1, 0, 0, BASS_WASAPI_AUTOFORMAT, 0.05, 0, WasapiProc, nil);
  CheckBassError;
  BASS_WASAPI_GetInfo(WI);
  CheckBassError;
  M := BASS_Mixer_StreamCreate(WI.freq, WI.chans, BASS_STREAM_DECODE or BASS_SAMPLE_FLOAT);
  CheckBassError;
  BASS_WASAPI_Start;
  CheckBassError;
end;

procedure TForm2.Panel1Resize(Sender: TObject);
begin
  //update video size
  BASS_ChannelSetAttribute(C, BASS_V2_ATTRIB_VIDEOSZ, Panel1.Handle);
end;

procedure TForm2.Timer1Timer(Sender: TObject);
var
  P: Single;
begin
  //show current pos
  P := BASS_ChannelBytes2Seconds(C, BASS_ChannelGetPosition(C, 0));
  Label1.Caption := Format('P=%.1fs'#13#10'L=%1fs',
    [P , Sec(BASS_ChannelGetLength(C, 0))]);
end;

procedure TForm2.TrackBar1Change(Sender: TObject);
begin
  //change pos
  BASS_ChannelSetPosition(C, TrackBar1.Position, 0);
end;

initialization
  MyDir := IncludeTrailingBackslash(ExtractFileDir(Application.ExeName));

end.
